@@ -34,6 +34,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#include <signal.h>
 #include <unistd.h>
 #include <time.h>
 #include <ctype.h>
@@ -95,6 +96,7 @@ static struct config {
     char *eval;
 } config;
 
+static volatile sig_atomic_t force_cancel_loop = 0;
 static void usage();
 static void slaveMode(void);
 char *redisGitSHA1(void);
@@ -1806,11 +1808,17 @@ unsigned long compute_something_fast(void) {
     return output;
 }
 
+static void intrinsicLatencyModeStop(int s) {
+    REDIS_NOTUSED(s);
+    force_cancel_loop = 1;
+}
+
 static void intrinsicLatencyMode(void) {
     long long test_end, run_time, max_latency = 0, runs = 0;
 
     run_time = config.intrinsic_latency_duration*1000000;
     test_end = ustime() + run_time;
+    signal(SIGINT, intrinsicLatencyModeStop);
 
     while(1) {
         long long start, end, latency;
@@ -1828,10 +1836,10 @@ static void intrinsicLatencyMode(void) {
             printf("Max latency so far: %lld microseconds.\n", max_latency);
         }
 
-        if (end > test_end) {
+        if (force_cancel_loop || end > test_end) {
             printf("\n%lld total runs (avg %lld microseconds per run).\n",
                 runs, run_time/runs);
-            printf("Worst run took %.02fx times the avarege.\n",
+            printf("Worst run took %.02fx times the average.\n",
                 (double) max_latency / (run_time/runs));
             exit(0);
         }
